/*
 * emulator_implementation.cpp
 *
 *  Created on: Oct 15, 2010
 *      Author: Squarepusher2 
 */

#ifdef PSGL
#include "cellframework/graphics/PSGLGraphics.h"
#else
#include "../old/cellgraphicsgcm.h"
#endif

#include "snes9x.h"
#include "memmap.h"
#include "apu/apu.h"
#include "gfx.h"
#include "snapshot.h"
#include "controls.h"
#include "cheats.h"
#include "movie.h"
#include "logger.h"
#include "display.h"
#include "conffile.h"

#include "emulator_implementation.h"
#include "emulator_graphics.h"

static ConfigFile	currentconfig;

void Emulator_Implementation_SaveSRAM()
{
	Memory.SaveSRAM(S9xGetFilename(".srm", SRAM_DIR));
}

bool Emulator_Implementation_InitSystem(Emulator_ControllerModes controller_setting)
{
	if(Emulator_IsInitialized())
	{
		Memory.Deinit();
	}
	Memory.Init();

	if(Emulator_IsInitialized)
	{
		S9xDeinitAPU();
	}
	S9xInitAPU();

	S9xInitSound(100, 25);

	S9xSetSoundMute(TRUE);
	S9xSetRenderPixelFormat(RGB555);

	if (Emulator_GetROM() == NULL)
	{
		Emulator_SetDebugMessage("No ROM to load");
		return false;
	}

	if (!Memory.LoadROM(Emulator_GetROM())) {
		Emulator_SetDebugMessage("Load ROM failed");
		return false;
	}

	Memory.LoadSRAM(S9xGetFilename(".srm", SRAM_DIR));

	GFX.Pitch = 1024;
	GFX.Screen = (uint16_t *)memalign(1024, 1024 * 478);

	if(Emulator_IsInitialized())
	{
		S9xGraphicsDeinit();
	}
	S9xGraphicsInit();

	// Controller initialization
	switch(controller_setting)
	{
		case TWO_JOYSTICKS:
			S9xSetController(0, CTL_JOYPAD, 0, 0, 0, 0);
			S9xSetController(1, CTL_JOYPAD, 1, 0, 0, 0);
			break;
		case MULTITAP:
			S9xSetController(0, CTL_JOYPAD,	0,0,0,0);
			S9xSetController(1, CTL_MP5,	1,2,3,4);
			break;

		default:
			break;
	}

	S9xUnmapAllControls();

#define MAP_BUTTON(id, name) S9xMapButton((id), S9xGetCommandT((name)), false)

	MAP_BUTTON(MAKE_BUTTON(PAD_1, BTN_A), "Joypad1 A");
	MAP_BUTTON(MAKE_BUTTON(PAD_1, BTN_B), "Joypad1 B");
	MAP_BUTTON(MAKE_BUTTON(PAD_1, BTN_X), "Joypad1 X");
	MAP_BUTTON(MAKE_BUTTON(PAD_1, BTN_Y), "Joypad1 Y");
	MAP_BUTTON(MAKE_BUTTON(PAD_1, BTN_SELECT), "Joypad1 Select");
	MAP_BUTTON(MAKE_BUTTON(PAD_1, BTN_START), "Joypad1 Start");
	MAP_BUTTON(MAKE_BUTTON(PAD_1, BTN_L), "Joypad1 L");
	MAP_BUTTON(MAKE_BUTTON(PAD_1, BTN_R), "Joypad1 R");
	MAP_BUTTON(MAKE_BUTTON(PAD_1, BTN_LEFT), "Joypad1 Left");
	MAP_BUTTON(MAKE_BUTTON(PAD_1, BTN_RIGHT), "Joypad1 Right");
	MAP_BUTTON(MAKE_BUTTON(PAD_1, BTN_UP), "Joypad1 Up");
	MAP_BUTTON(MAKE_BUTTON(PAD_1, BTN_DOWN), "Joypad1 Down");
	MAP_BUTTON(MAKE_BUTTON(PAD_1, BTN_QUICKSAVE), "SaveFreezeFile");
	MAP_BUTTON(MAKE_BUTTON(PAD_1, BTN_QUICKLOAD), "LoadFreezeFile");
	MAP_BUTTON(MAKE_BUTTON(PAD_1, BTN_INCREMENTSAVE), "IncSave");
	MAP_BUTTON(MAKE_BUTTON(PAD_1, BTN_DECREMENTSAVE), "DecSave");
	MAP_BUTTON(MAKE_BUTTON(PAD_2, BTN_A), "Joypad2 A");
	MAP_BUTTON(MAKE_BUTTON(PAD_2, BTN_B), "Joypad2 B");
	MAP_BUTTON(MAKE_BUTTON(PAD_2, BTN_X), "Joypad2 X");
	MAP_BUTTON(MAKE_BUTTON(PAD_2, BTN_Y), "Joypad2 Y");
	MAP_BUTTON(MAKE_BUTTON(PAD_2, BTN_SELECT), "Joypad2 Select");
	MAP_BUTTON(MAKE_BUTTON(PAD_2, BTN_START), "Joypad2 Start");
	MAP_BUTTON(MAKE_BUTTON(PAD_2, BTN_L), "Joypad2 L");
	MAP_BUTTON(MAKE_BUTTON(PAD_2, BTN_R), "Joypad2 R");
	MAP_BUTTON(MAKE_BUTTON(PAD_2, BTN_LEFT), "Joypad2 Left");
	MAP_BUTTON(MAKE_BUTTON(PAD_2, BTN_RIGHT), "Joypad2 Right");
	MAP_BUTTON(MAKE_BUTTON(PAD_2, BTN_UP), "Joypad2 Up");
	MAP_BUTTON(MAKE_BUTTON(PAD_2, BTN_DOWN), "Joypad2 Down");
	MAP_BUTTON(MAKE_BUTTON(PAD_2, BTN_QUICKSAVE), "SaveFreezeFile");
	MAP_BUTTON(MAKE_BUTTON(PAD_2, BTN_QUICKLOAD), "LoadFreezeFile");
	MAP_BUTTON(MAKE_BUTTON(PAD_2, BTN_INCREMENTSAVE), "IncSave");
	MAP_BUTTON(MAKE_BUTTON(PAD_2, BTN_DECREMENTSAVE), "DecSave");
	switch(controller_setting)
	{
		case MULTITAP:
			//player 3 - for multitap
			MAP_BUTTON(MAKE_BUTTON(PAD_3, BTN_A), "Joypad3 A");
			MAP_BUTTON(MAKE_BUTTON(PAD_3, BTN_B), "Joypad3 B");
			MAP_BUTTON(MAKE_BUTTON(PAD_3, BTN_X), "Joypad3 X");
			MAP_BUTTON(MAKE_BUTTON(PAD_3, BTN_Y), "Joypad3 Y");
			MAP_BUTTON(MAKE_BUTTON(PAD_3, BTN_SELECT), "Joypad3 Select");
			MAP_BUTTON(MAKE_BUTTON(PAD_3, BTN_START), "Joypad3 Start");
			MAP_BUTTON(MAKE_BUTTON(PAD_3, BTN_L), "Joypad3 L");
			MAP_BUTTON(MAKE_BUTTON(PAD_3, BTN_R), "Joypad3 R");
			MAP_BUTTON(MAKE_BUTTON(PAD_3, BTN_LEFT), "Joypad3 Left");
			MAP_BUTTON(MAKE_BUTTON(PAD_3, BTN_RIGHT), "Joypad3 Right");
			MAP_BUTTON(MAKE_BUTTON(PAD_3, BTN_UP), "Joypad3 Up");
			MAP_BUTTON(MAKE_BUTTON(PAD_3, BTN_DOWN), "Joypad3 Down");
			MAP_BUTTON(MAKE_BUTTON(PAD_3, BTN_QUICKSAVE), "SaveFreezeFile");
			MAP_BUTTON(MAKE_BUTTON(PAD_3, BTN_QUICKLOAD), "LoadFreezeFile");
			MAP_BUTTON(MAKE_BUTTON(PAD_3, BTN_INCREMENTSAVE), "IncSave");
			MAP_BUTTON(MAKE_BUTTON(PAD_3, BTN_DECREMENTSAVE), "DecSave");
			//player 4 - for multitap
			MAP_BUTTON(MAKE_BUTTON(PAD_4, BTN_A), "Joypad4 A");
			MAP_BUTTON(MAKE_BUTTON(PAD_4, BTN_B), "Joypad4 B");
			MAP_BUTTON(MAKE_BUTTON(PAD_4, BTN_X), "Joypad4 X");
			MAP_BUTTON(MAKE_BUTTON(PAD_4, BTN_Y), "Joypad4 Y");
			MAP_BUTTON(MAKE_BUTTON(PAD_4, BTN_SELECT), "Joypad4 Select");
			MAP_BUTTON(MAKE_BUTTON(PAD_4, BTN_START), "Joypad4 Start");
			MAP_BUTTON(MAKE_BUTTON(PAD_4, BTN_L), "Joypad4 L");
			MAP_BUTTON(MAKE_BUTTON(PAD_4, BTN_R), "Joypad4 R");
			MAP_BUTTON(MAKE_BUTTON(PAD_4, BTN_LEFT), "Joypad4 Left");
			MAP_BUTTON(MAKE_BUTTON(PAD_4, BTN_RIGHT), "Joypad4 Right");
			MAP_BUTTON(MAKE_BUTTON(PAD_4, BTN_UP), "Joypad4 Up");
			MAP_BUTTON(MAKE_BUTTON(PAD_4, BTN_DOWN), "Joypad4 Down");
			MAP_BUTTON(MAKE_BUTTON(PAD_4, BTN_QUICKSAVE), "SaveFreezeFile");
			MAP_BUTTON(MAKE_BUTTON(PAD_4, BTN_QUICKLOAD), "LoadFreezeFile");
			MAP_BUTTON(MAKE_BUTTON(PAD_4, BTN_INCREMENTSAVE), "IncSave");
			MAP_BUTTON(MAKE_BUTTON(PAD_4, BTN_DECREMENTSAVE), "DecSave");
			//player 5 - for multitap
			MAP_BUTTON(MAKE_BUTTON(PAD_5, BTN_A), "Joypad5 A");
			MAP_BUTTON(MAKE_BUTTON(PAD_5, BTN_B), "Joypad5 B");
			MAP_BUTTON(MAKE_BUTTON(PAD_5, BTN_X), "Joypad5 X");
			MAP_BUTTON(MAKE_BUTTON(PAD_5, BTN_Y), "Joypad5 Y");
			MAP_BUTTON(MAKE_BUTTON(PAD_5, BTN_SELECT), "Joypad5 Select");
			MAP_BUTTON(MAKE_BUTTON(PAD_5, BTN_START), "Joypad5 Start");
			MAP_BUTTON(MAKE_BUTTON(PAD_5, BTN_L), "Joypad5 L");
			MAP_BUTTON(MAKE_BUTTON(PAD_5, BTN_R), "Joypad5 R");
			MAP_BUTTON(MAKE_BUTTON(PAD_5, BTN_LEFT), "Joypad5 Left");
			MAP_BUTTON(MAKE_BUTTON(PAD_5, BTN_RIGHT), "Joypad5 Right");
			MAP_BUTTON(MAKE_BUTTON(PAD_5, BTN_UP), "Joypad5 Up");
			MAP_BUTTON(MAKE_BUTTON(PAD_5, BTN_DOWN), "Joypad5 Down");
			MAP_BUTTON(MAKE_BUTTON(PAD_5, BTN_QUICKSAVE), "SaveFreezeFile");
			MAP_BUTTON(MAKE_BUTTON(PAD_5, BTN_QUICKLOAD), "LoadFreezeFile");
			MAP_BUTTON(MAKE_BUTTON(PAD_5, BTN_INCREMENTSAVE), "IncSave");
			MAP_BUTTON(MAKE_BUTTON(PAD_5, BTN_DECREMENTSAVE), "DecSave");
			break;

		default:
			break;
	}
	
	return true;	
}

void Emulator_Implementation_Inputloop()
{
	uint8_t pads_connected = CellInput->NumberPadsConnected();
	for (uint8_t i = 0; i < pads_connected; ++i)
	{
		if(CellInput->UpdateDevice(i) != CELL_PAD_OK)
		{
			continue;
		}
		unsigned int pad_id = i + 1;
		S9xReportButton(MAKE_BUTTON(pad_id, BTN_A), (CellInput->IsButtonPressed(i,CTRL_CIRCLE)));
		S9xReportButton(MAKE_BUTTON(pad_id, BTN_B), (CellInput->IsButtonPressed(i,CTRL_CROSS)));
		S9xReportButton(MAKE_BUTTON(pad_id, BTN_X), (CellInput->IsButtonPressed(i,CTRL_TRIANGLE)));
		S9xReportButton(MAKE_BUTTON(pad_id, BTN_Y), (CellInput->IsButtonPressed(i,CTRL_SQUARE)));
		S9xReportButton(MAKE_BUTTON(pad_id, BTN_SELECT), (CellInput->IsButtonPressed(i,CTRL_SELECT)));
		S9xReportButton(MAKE_BUTTON(pad_id, BTN_START), (CellInput->IsButtonPressed(i,CTRL_START)));
		S9xReportButton(MAKE_BUTTON(pad_id, BTN_L), (CellInput->IsButtonPressed(i,CTRL_L1)));
		S9xReportButton(MAKE_BUTTON(pad_id, BTN_R), (CellInput->IsButtonPressed(i,CTRL_R1)));
		S9xReportButton(MAKE_BUTTON(pad_id, BTN_UP), (CellInput->IsButtonPressed(i,CTRL_UP) | CellInput->IsAnalogPressedUp(i,CTRL_LSTICK)));
		S9xReportButton(MAKE_BUTTON(pad_id, BTN_DOWN), (CellInput->IsButtonPressed(i,CTRL_DOWN) | CellInput->IsAnalogPressedDown(i,CTRL_LSTICK)));
		S9xReportButton(MAKE_BUTTON(pad_id, BTN_LEFT), (CellInput->IsButtonPressed(i,CTRL_LEFT) | CellInput->IsAnalogPressedLeft(i,CTRL_LSTICK)));
		S9xReportButton(MAKE_BUTTON(pad_id, BTN_RIGHT), (CellInput->IsButtonPressed(i,CTRL_RIGHT) | CellInput->IsAnalogPressedRight(i,CTRL_LSTICK)));
		S9xReportButton(MAKE_BUTTON(pad_id, BTN_QUICKSAVE), (CellInput->IsButtonPressed(i,CTRL_R2) & CellInput->IsButtonPressed(i,CTRL_R3)));
		S9xReportButton(MAKE_BUTTON(pad_id, BTN_QUICKLOAD), (CellInput->IsButtonPressed(i,CTRL_L2) & CellInput->IsButtonPressed(i,CTRL_R3)));
		S9xReportButton(MAKE_BUTTON(pad_id, BTN_INCREMENTSAVE), (CellInput->WasAnalogPressedRight(i,CTRL_RSTICK)));
		S9xReportButton(MAKE_BUTTON(pad_id, BTN_DECREMENTSAVE), (CellInput->WasAnalogPressedLeft(i,CTRL_RSTICK)));
		if(CellInput->IsButtonPressed(i,CTRL_R3) & CellInput->IsButtonPressed(i,CTRL_L3))
		{
			Emulator_StopROMRunning();
			Emulator_SwitchMode(MODE_MENU);
		}
	}
}

bool Emulator_Implementation_InitSettings()
{
	ZeroMemory(&Settings, sizeof(Settings));

	/*
	Settings.DumpStreamsMaxFrames = -1;
	Settings.CartAName[0] = 0;
	Settings.CartBName[0] = 0;
	*/

	S9xLoadConfigFiles();

	return true;
}

bool Emulator_Implementation_SaveSettings()
{
	currentconfig.SetBool("PS3General::KeepAspect",Settings.PS3KeepAspect);
	currentconfig.SetBool("PS3General::Smooth", Settings.PS3Smooth);
	currentconfig.SetBool("PS3General::OverscanEnabled", Settings.PS3OverscanEnabled);
	currentconfig.SetInt("PS3General::OverscanAmount",Settings.PS3OverscanAmount);
	currentconfig.SetString("PS3Paths::PathSaveStates",Settings.PS3PathSaveStates);
	currentconfig.SetString("PS3Paths::PathROMDirectory",Settings.PS3PathROMDirectory);
	currentconfig.SetString("RSound::RSoundServerIPAddress",Settings.RSoundServerIPAddress);
	currentconfig.SetBool("RSound::RSoundEnabled",Settings.RSoundEnabled);
	currentconfig.SetUInt("Sound::InputRate",Settings.SoundInputRate);
	currentconfig.SetBool("Sound::Sync",Settings.SoundSync);
	currentconfig.SetBool("ROM::PAL",Settings.ForcePAL);
	currentconfig.SetBool("ROM::NTSC",Settings.ForceNTSC);
	currentconfig.SetBool("Display::DisplayFrameRate",Settings.DisplayFrameRate);
	currentconfig.SetBool("Display::GraphicWindows",!Settings.DisableGraphicWindows);
	currentconfig.SetBool("Display::Transparency",Settings.Transparency);
	currentconfig.SetBool("Display::DisplayInput",Settings.DisplayPressedKeys);
	currentconfig.SetBool("Display::MessagesInImage",Settings.AutoDisplayMessages);
	currentconfig.SetBool("Settings::BSXBootup",Settings.BSXBootup);
	return currentconfig.SaveTo(SYS_CONFIG_FILE);
}

void S9xParsePortConfig (ConfigFile &conf, int pass)
{
	currentconfig = conf;
	if (pass == 1)
	{
		//Some additional settings that are not properly set by SNES9x's LoadConfigFiles
		if (currentconfig.Exists("Settings::FrameTimePAL"))
		{
			Settings.FrameTimePAL = conf.GetUInt("Settings::FrameTimePAL", 20000);
		}
		//PS3 - General Settings
		if (currentconfig.Exists("PS3General::KeepAspect"))
		{
			Settings.PS3KeepAspect		= currentconfig.GetBool("PS3General::KeepAspect");
		}
		else
		{
			Settings.PS3KeepAspect		= TRUE;
		}
		if (currentconfig.Exists("PS3General::Smooth"))
		{
			Settings.PS3Smooth		= currentconfig.GetBool("PS3General::Smooth");
		}
		else
		{
			Settings.PS3Smooth		= TRUE;
		}
		if (currentconfig.Exists("PS3General::OverscanEnabled"))
		{
			Settings.PS3OverscanEnabled	= currentconfig.GetBool("PS3General::OverscanEnabled");
		}
		else
		{
			Settings.PS3OverscanEnabled	= FALSE;
		}
		if (currentconfig.Exists("PS3General::OverscanAmount"))
		{
			Settings.PS3OverscanAmount	= currentconfig.GetInt("PS3General::OverscanAmount");
		}
		else
		{
			Settings.PS3OverscanAmount	= 0;
		}
		//RSound Settings
		if(currentconfig.Exists("RSound::RSoundEnabled"))
		{
			Settings.RSoundEnabled		= currentconfig.GetBool("RSound::RSoundEnabled");
		}
		else
		{
			Settings.RSoundEnabled		= FALSE;
		}
		if(currentconfig.Exists("RSound::RSoundServerIPAddress"))
		{
			Settings.RSoundServerIPAddress	= currentconfig.GetString("RSound::RSoundServerIPAddress");
		}
		else
		{
			Settings.RSoundServerIPAddress = "0.0.0.0";
		}
		
		// PS3 Path Settings
		 if ((currentconfig.Exists("PS3Paths::PathSaveStates")) && strlen(currentconfig.GetString("PS3Paths::PathSaveStates")) > 0)
		 {
		 Settings.PS3PathSaveStates		= currentconfig.GetString("PS3Paths::PathSaveStates");
		 }
		 else
		 {
		 Settings.PS3PathSaveStates		= USRDIR;
		 }
		 
		 /*
		 if (currentconfig.Exists("PS3Paths::PathSRAM"))
		 {
		 Settings.PS3PathSRAM		= currentconfig.GetString("PS3Paths::PathSRAM");
		 }
		
		 if (currentconfig.Exists("PS3Paths::PathScreenshots"))
		 {
		 Settings.PS3PathScreenshots		= currentconfig.GetString("PS3Paths::PathScreenshots");
		 }
		 */
		 
		 if (currentconfig.Exists("PS3Paths::PathROMDirectory"))
		 {
			 Settings.PS3PathROMDirectory		= currentconfig.GetString("PS3Paths::PathROMDirectory");
		 }
		 else
		 {
			 Settings.PS3PathROMDirectory	= "/";
		 }
	}
}

void S9xExtraUsage (void)
{
}

void S9xParseArg (char **argv, int &i, int argc)
{
}

bool S9xPollAxis (uint32_t id, int16_t *value)
{
	return false;
}

bool S9xPollButton (uint32_t id, bool *pressed)
{
	return false;
}

bool S9xPollPointer (uint32_t id, int16_t *x, int16_t *y)
{
	return false;
}

void S9xHandlePortCommand (s9xcommand_t cmd, int16_t data1, int16_t data2)
{
}


const char *S9xStringInput (const char *message)
{
	return NULL;
}

bool8 S9xOpenSnapshotFile (const char *filepath, bool8 read_only, STREAM *file)
{
	if(read_only)
	{
		if((*file = OPEN_STREAM(filepath, "rb")) != 0)
		{
			return (TRUE);
		}
	}
	else
	{
		if((*file = OPEN_STREAM(filepath, "wb")) != 0)
		{
			return (TRUE);
		}
	}
	return (FALSE);
}

void S9xCloseSnapshotFile (STREAM file)
{
	CLOSE_STREAM (file);
}

void S9xExit (void)
{
}

bool8 S9xInitUpdate (void)
{
	return true;
}

bool8 S9xDeinitUpdate (int width, int height)
{
#ifdef PSGL
   glClear(GL_COLOR_BUFFER_BIT);

   glBufferSubData(GL_TEXTURE_REFERENCE_BUFFER_SCE, 0, height * SNES9X_SCREEN_PITCH, GFX.Screen);
   glTextureReferenceSCE(GL_TEXTURE_2D, 1, width, height, 0, GL_RGB5_A1, SCREEN_RENDER_TEXTURE_PITCH, 0);

   glDrawArrays(GL_QUADS, 0, 4); 
   glFlush();

   //write_fps();

   //psglSwap(); // Swaps the render to screen.
   return true;
#else
	unsigned int baseX = 0;
	unsigned int baseY = 0;
	int xscale;
	int yscale;
	int scale;
	unsigned int destW, destH;
	unsigned int xinc = 0;
	unsigned int yinc = 0;
	uint16_t *gfxptr;
    	uint32_t *displayptr;

    // find the scale between the current ps3 resolution and the desired snes9x one
	xscale = color_buf[0].width / width;
	yscale = color_buf[0].height / height;

	// Function debug line - WARNING emulation is a slide show
	//EMU_DBG("S9xDeinitUpdate(%d, %d) & \n ps3Res(%d, %d, %d) & \n scale(%d, %d)", width, height, CellGetVideoState().displayMode.resolutionId, color_buf[0].width, color_buf[0].height, xscale, yscale);

	// this code does not check out with how to scale
	// FIXME: verify this later, fixes most broken resolution games
	if (Graphics_GetVideoState().displayMode.resolutionId == CELL_VIDEO_OUT_RESOLUTION_720 ||
			Graphics_GetVideoState().displayMode.resolutionId == CELL_VIDEO_OUT_RESOLUTION_1080)
	{
		if (xscale > yscale)
			xscale = yscale;
		else if (yscale > xscale)
			yscale = xscale;
	}

	// calculate the end point of the screen (bottom right)
	destW = (width * xscale);
	destH = (height * yscale);

	// calculate the X,Y offset to center the screen in pS3 resolution
	baseX = color_buf[0].width/2 - destW/2;
	baseY = color_buf[0].height/2 - destH/2;

	// display is a 1D array representing 2D pixels
	// this gets us an offset to the correct index to begin writing in based on the baseX/Y
	displayptr = (uint32_t *)&display[baseY * color_buf[0].width + baseX];

	// get the index 0 the snes9x screen
	gfxptr = GFX.Screen;

	for (unsigned int y = 0; y < destH; ++y)
	{
		for (unsigned int x = 0; x < destW; ++x)
		{
			// extract the RBG value from snes9x screen
			*(displayptr++) = ((*gfxptr & 0x1f) << 3) | ((*gfxptr & 0x7e0) << 5) | ((*gfxptr & 0xf800) << 8);

			// to scale the x we need to copy xscale pixels before incrementing
			// once we have copied in xscale pixels from the same gfxptr index move to the next x pixel
			if (++xinc == xscale)
			{
				gfxptr++;
				xinc = 0;
			}
		}

		// same deal as above for xscale.
		// except here we jump an entire row
		if (++yinc == yscale)
		{
			gfxptr += (GFX.Pitch>>1) - width;
			yinc = 0;
		}
		else
		{
			// need to repeat a row, so move gfxptr back to the same row
			gfxptr -= width;
		}

		// increment the ps3 render buffer by an entire line
		displayptr += color_buf[0].width - destW;
	}
	return true;
#endif
}

bool8 S9xContinueUpdate (int width, int height) {

   //return S9xDeinitUpdate(width, height);
   return true;
}

void S9xMessage (int type, int number, char const *message)
{
	S9xSetInfoString(message);
}

bool8 S9xOpenSoundDevice (void) {
	return true;
}

const char * S9xGetFilename (const char *ex, enum s9x_getdirtype dirtype)
{
	static char	s[PATH_MAX + 1];
	char		drive[_MAX_DRIVE + 1];
	char		dir[_MAX_DIR + 1];
	char		fname[_MAX_FNAME + 1];
	char		ext[_MAX_EXT + 1];

	_splitpath(Memory.ROMFilename, drive, dir, fname, ext);
	snprintf(s, PATH_MAX + 1, "%s%s%s%s", S9xGetDirectory(dirtype), SLASH_STR, fname, ex);

	return (s);
}

const char * S9xGetFilenameInc (const char *ex, enum s9x_getdirtype dirtype)
{
	static char	s[PATH_MAX + 1];
	char		drive[_MAX_DRIVE + 1];
	char		dir[_MAX_DIR + 1];
	char		fname[_MAX_FNAME + 1];
	char		ext[_MAX_EXT + 1];

	unsigned int	i = 0;
	const char	*d;

	_splitpath(Memory.ROMFilename, drive, dir, fname, ext);
	d = S9xGetDirectory(dirtype);

	snprintf(s, PATH_MAX + 1, "%s%s%s.%03d%s", d, SLASH_STR, fname, i++, ex);

	return (s);
}

const char * S9xBasename (const char *f)
{
	const char	*p;

	if ((p = strrchr(f, '/')) != NULL || (p = strrchr(f, '\\')) != NULL)
	{
		return (p + 1);
	}

	return (f);
}

const char *S9xGetDirectory (enum s9x_getdirtype dirtype) {
	static char s[PATH_MAX + 1];

	switch (dirtype)
	{
		case SNAPSHOT_DIR:
			return Settings.PS3PathSaveStates;
			break;
		case SRAM_DIR:
		case ROMFILENAME_DIR:
			strncpy(s, Memory.ROMFilename, PATH_MAX + 1);
			s[PATH_MAX] = 0;

			for (unsigned int i = strlen(s); i >= 0; --i)
			{
				if (s[i] == SLASH_CHAR)
				{
					s[i] = 0;
					break;
				}
			}

			return s;
			break;
		default:
			return USRDIR;
	}
}

const char *S9xChooseFilename (bool8 read_only)
{
	static char	filename[PATH_MAX + 1];
	static char	drive[_MAX_DRIVE + 1];
	static char	dir[_MAX_DIR + 1];
	static char	def[_MAX_FNAME + 1];
	static char	ext[_MAX_EXT + 1];

	_splitpath(Memory.ROMFilename, drive, dir, def, ext);
	snprintf(filename, PATH_MAX + 1, "%s%s%s.%03d", S9xGetDirectory(SNAPSHOT_DIR), SLASH_STR, def, Emulator_CurrentSaveStateSlot());

	return filename;
}

const char *S9xChooseMovieFilename (bool8 read_only)
{
	return NULL;
}

void S9xAutoSaveSRAM (void)
{
	Memory.SaveSRAM(S9xGetFilename(".srm", SRAM_DIR));
}

void S9xToggleSoundChannel (int c)
{
}

void S9xSetPalette (void)
{
}

void S9xSyncSpeed (void)
{
}


void _splitpath (const char *path, char *drive, char *dir, char *fname, char *ext)
{
	*drive = 0;

	const char	*slash = strrchr(path, SLASH_CHAR);
	const char	*dot   = strrchr(path, '.');

	if (dot && slash && dot < slash)
	{
		dot = NULL;
	}

	if (!slash)
	{
		*dir = 0;

		strcpy(fname, path);

		if (dot)
		{
			fname[dot - path] = 0;
			strcpy(ext, dot + 1);
		}
		else
		{
			*ext = 0;
		}
	}
	else
	{
		strcpy(dir, path);
		dir[slash - path] = 0;

		strcpy(fname, slash + 1);

		if (dot)
		{
			fname[dot - slash - 1] = 0;
			strcpy(ext, dot + 1);
		}
		else
		{
			*ext = 0;
		}
	}
}

void _makepath (char *path, const char *, const char *dir, const char *fname, const char *ext)
{
	if (dir && *dir)
	{
		strcpy(path, dir);
		strcat(path, SLASH_STR);
	}
	else
	{
		*path = 0;
	}

	strcat(path, fname);

	if (ext && *ext)
	{
		strcat(path, ".");
		strcat(path, ext);
	}
}
