/*
 * menu.cpp
 *
 *  Created on: Oct 10, 2010
 *      Author: halsafar
 */
#include <string.h>
#include <stack>
#include <vector>

#include "snes9x.h"

#include "emulator.h"
#include "emulator_graphics.h"
//#include "emulator_audio.h"
#include "menu.h"
#include "cellframework/input/cellInput.h"

#include <cell/pad.h>

#include <cell/audio.h>
#include <cell/sysmodule.h>
#include <cell/cell_fs.h>
#include <cell/dbgfont.h>
#include <sysutil/sysutil_sysparam.h>

#define MIN(x,y) ((x) < (y) ? (x) : (y))
#define MAX(x,y) ((x) > (y) ? (x) : (y))

enum MENU_RESOLUTION_CHOICES
{
	RESOLUTION_480,
	RESOLUTION_720,
	RESOLUTION_1080
};

static char rom_path[MAX_PATH];
static char* path;
// initial resolution
static uint8_t initial_resolution = 0;

// is the menu running
bool menuRunning = false;

// menu to render
typedef void (*curMenuPtr)();

//
std::stack<curMenuPtr> menuStack;


void MenuStop()
{
	menuRunning = false;

	while (!menuStack.empty())
	{
		menuStack.pop();
	}
}


bool MenuIsRunning()
{
	return menuRunning;
}


char* MenuGetSelectedROM()
{
	return rom_path;
}


char* MenuGetCurrentPath()
{
	return path;
}


void qsort_structs(CellFsDirent entries[], int left, int right)
{
	char *tmp_name;
	CellFsDirent temp_struct;
	register int x, y;

	x = left;
	y = right;
	tmp_name = entries[(left+right)/2].d_name;

	do
	{
		while((strcasecmp(entries[x].d_name, tmp_name) < 0) && (x < right))
		{
			x++;
		}
		while((strcasecmp(entries[y].d_name, tmp_name) > 0) && (y > left))
		{
			y--;
		}

		if(x <= y)
		{
			temp_struct = entries[x];
			entries[x] = entries[y];
			entries[y] = temp_struct;
			x++;
			y--;
		}
	}
	while (x <= y);

	if(left < y)
	{
		qsort_structs(entries, left, y);
	}

	if(x < right)
	{
		qsort_structs(entries, x, right);
	}
}

// void do_path_settings()
// // called from ROM menu by pressing the SELECT button
// // return to ROM menu by pressing the CIRCLE button
int currently_selected_path_setting = 0;
void do_path_settings()
{
	CellInput->UpdateDevice(0);
			// back to ROM menu if CIRCLE is pressed
			if (CellInput->WasButtonPressed(0, CTRL_L1) | CellInput->WasButtonPressed(0, CTRL_CIRCLE))
			{
				menuStack.pop();
				return;
			}

			/*
			if (CellInput->WasButtonPressed(0, CTRL_R1))
			{
				menuStack.push(do_path_settings);
				return;
			}
			*/

			if (CellInput->WasButtonPressed(0, CTRL_DOWN) | CellInput->WasAnalogPressedDown(0, CTRL_LSTICK))	// down to next setting
			{
				currently_selected_path_setting++;
				if (currently_selected_path_setting >= MAX_NO_OF_PATH_SETTINGS)
				{
					currently_selected_path_setting = 0;
				}
			}

			if (CellInput->WasButtonPressed(0, CTRL_UP) | CellInput->WasAnalogPressedUp(0, CTRL_LSTICK))	// up to previous setting
			{
					currently_selected_path_setting--;
					if (currently_selected_path_setting < 0)
					{
						currently_selected_path_setting = MAX_NO_OF_PATH_SETTINGS-1;
					}
			}
					switch(currently_selected_path_setting)
					{
						case SETTING_PATH_DEFAULT_ROM_DIRECTORY:
						if(CellInput->WasButtonPressed(0, CTRL_LEFT) | CellInput->WasAnalogPressedLeft(0, CTRL_LSTICK) | CellInput->WasButtonPressed(0, CTRL_RIGHT) | CellInput->WasAnalogPressedRight(0, CTRL_LSTICK) | CellInput->WasButtonPressed(0, CTRL_CROSS))
						{
							char * romdirectorypath = do_pathmenu();
							Settings.PS3PathROMDirectory = romdirectorypath;
						}
							break;
						case SETTING_PATH_SAVESTATES_DIRECTORY:
						if(CellInput->WasButtonPressed(0, CTRL_LEFT) | CellInput->WasAnalogPressedLeft(0, CTRL_LSTICK) | CellInput->WasButtonPressed(0, CTRL_RIGHT) | CellInput->WasAnalogPressedRight(0, CTRL_LSTICK) | CellInput->WasButtonPressed(0, CTRL_CROSS))
						{
							char * savestatepath = do_pathmenu();
							Settings.PS3PathSaveStates = savestatepath;
						}
							break;
						case SETTING_PATH_DEFAULT_ALL:
						if(CellInput->WasButtonPressed(0, CTRL_LEFT) | CellInput->WasAnalogPressedLeft(0, CTRL_LSTICK) | CellInput->WasButtonPressed(0, CTRL_RIGHT) | CellInput->WasAnalogPressedRight(0, CTRL_LSTICK) | CellInput->WasButtonPressed(0, CTRL_CROSS))
						{
							Settings.PS3PathROMDirectory = "/";
							Settings.PS3PathSaveStates = USRDIR;
						}
							break;
					break;
				default:
					break;
			} // end of switch

	//zero the screen buffer 
	Graphics_ClearSurface();

	cellDbgFontPuts		(0.05f,	0.05f,	FONT_SIZE,	GREEN,	"GENERAL");
	Graphics_RenderDbgFont();
	cellDbgFontPuts		(0.25f,	0.05f,	FONT_SIZE,	GREEN,	"SNES9x");
	Graphics_RenderDbgFont();
	cellDbgFontPuts		(0.45f,	0.05f,	FONT_SIZE,	RED,	"PATHS");
	Graphics_RenderDbgFont();

	cellDbgFontPuts		(0.05f,	0.09f,	FONT_SIZE,	currently_selected_path_setting == SETTING_PATH_DEFAULT_ROM_DIRECTORY ? YELLOW : WHITE,	"Startup ROM Directory");
	Graphics_RenderDbgFont();
	cellDbgFontPuts		(0.5f,	0.09f,	FONT_SIZE,	Settings.PS3PathROMDirectory == "/" ? GREEN : RED, Settings.PS3PathROMDirectory);
	Graphics_RenderDbgFont();

	cellDbgFontPuts		(0.05f,	0.13f,	FONT_SIZE,	currently_selected_path_setting == SETTING_PATH_SAVESTATES_DIRECTORY ? YELLOW : WHITE,	"Savestate Directory");
	Graphics_RenderDbgFont();
	cellDbgFontPuts		(0.5f,	0.13f,	FONT_SIZE,	Settings.PS3PathSaveStates == USRDIR ? GREEN : RED, Settings.PS3PathSaveStates);
	Graphics_RenderDbgFont();

	cellDbgFontPrintf(0.05f, 0.17f, FONT_SIZE, currently_selected_path_setting == SETTING_PATH_DEFAULT_ALL ? YELLOW : GREEN, "DEFAULT");


	cellDbgFontPuts(0.01f, 0.92f, FONT_SIZE, YELLOW, "UP/DOWN - select  LEFT/RIGHT - change  START - default  L1/CIRCLE - go back");
	Graphics_RenderDbgFont();


	cellSysutilCheckCallback();
	Graphics_Render(true);
}

// void do_general_settings()
// // called from ROM menu by pressing the SELECT button
// // return to ROM menu by pressing the CIRCLE button
int currently_selected_snes9x_setting = 0;
void do_snes9x_settings()
{
	CellInput->UpdateDevice(0);
			// back to ROM menu if CIRCLE is pressed
			if (CellInput->WasButtonPressed(0, CTRL_L1) | CellInput->WasButtonPressed(0, CTRL_CIRCLE))
			{
				menuStack.pop();
				return;
			}

			if (CellInput->WasButtonPressed(0, CTRL_R1))
			{
				menuStack.push(do_path_settings);
				return;
			}

			if (CellInput->WasButtonPressed(0, CTRL_DOWN) | CellInput->WasAnalogPressedDown(0, CTRL_LSTICK))	// down to next setting
			{
				currently_selected_snes9x_setting++;
				if (currently_selected_snes9x_setting >= MAX_NO_OF_SNES9X_SETTINGS)
				{
					currently_selected_snes9x_setting = 0;
				}
			}
			if (CellInput->WasButtonPressed(0, CTRL_UP) | CellInput->WasAnalogPressedUp(0, CTRL_LSTICK))	// up to previous setting
			{
					currently_selected_snes9x_setting--;
					if (currently_selected_snes9x_setting < 0)
					{
						currently_selected_snes9x_setting = MAX_NO_OF_SNES9X_SETTINGS-1;
					}
			}

					switch(currently_selected_snes9x_setting)
					{
					// display framerate on/off
					case SETTING_DISPLAY_FRAMERATE:
						if(CellInput->WasButtonPressed(0, CTRL_LEFT) | CellInput->WasAnalogPressedLeft(0, CTRL_LSTICK) | CellInput->WasButtonPressed(0, CTRL_RIGHT) | CellInput->WasAnalogPressedRight(0, CTRL_LSTICK) | CellInput->WasButtonPressed(0, CTRL_CROSS))
						{
							Settings.DisplayFrameRate = !Settings.DisplayFrameRate;
						}
						if(CellInput->IsButtonPressed(0, CTRL_START))
						{
							Settings.DisplayFrameRate = FALSE;
						}
						break;
					case SETTING_SOUND_INPUT_RATE:
						if(CellInput->WasButtonPressed(0, CTRL_LEFT) | CellInput->WasAnalogPressedLeft(0,CTRL_LSTICK))
						{
							if(Settings.SoundInputRate > 30000)
							{
								Settings.SoundInputRate--;
							}
						}
						if(CellInput->WasButtonPressed(0, CTRL_RIGHT) | CellInput->WasAnalogPressedRight(0,CTRL_LSTICK) | CellInput->WasButtonPressed(0,CTRL_CROSS))
						{
							if(Settings.SoundInputRate < 32000) // SNES APU has an input rate of 32000, no higher
							{
								Settings.SoundInputRate++;
							}
						}
						if(CellInput->IsButtonPressed(0, CTRL_START))
						{
							Settings.SoundInputRate = 31942;
						}
						break;
					case SETTING_TRANSPARENCY:
						if(CellInput->WasButtonPressed(0, CTRL_LEFT) | CellInput->WasAnalogPressedLeft(0,CTRL_LSTICK) | CellInput->WasButtonPressed(0, CTRL_RIGHT) | CellInput->WasAnalogPressedRight(0,CTRL_LSTICK) | CellInput->WasButtonPressed(0, CTRL_CROSS))
						{
							Settings.Transparency = !Settings.Transparency;
						}
						if(CellInput->IsButtonPressed(0, CTRL_START))
						{
							Settings.Transparency = TRUE;
						}
						break;
					case SETTING_SKIP_FRAMES:
						   if(CellInput->WasButtonPressed(0, CTRL_LEFT) | CellInput->WasAnalogPressedLeft(0,CTRL_LSTICK))
						   {
							   if (Settings.SkipFrames <= 1)
							   {
								   Settings.SkipFrames = AUTO_FRAMERATE;
							   }
							   else if(Settings.SkipFrames != AUTO_FRAMERATE)
							   {
								   Settings.SkipFrames--;
							   }
						   }
						   if(CellInput->WasButtonPressed(0, CTRL_RIGHT) | CellInput->WasAnalogPressedRight(0,CTRL_LSTICK) | CellInput->WasButtonPressed(0, CTRL_CROSS))
						   {
							   if (Settings.SkipFrames == AUTO_FRAMERATE)
							   {
								   Settings.SkipFrames = 1;
							   }
							   else if (Settings.SkipFrames < 10)
							   {
								   Settings.SkipFrames++;
							   }
						   }
						   if(CellInput->IsButtonPressed(0, CTRL_START))
						   {
							   Settings.SkipFrames = AUTO_FRAMERATE;
						   }
						   break;
					case SETTING_DISABLE_GRAPHIC_WINDOWS:
						   if(CellInput->WasButtonPressed(0, CTRL_RIGHT) | CellInput->WasAnalogPressedRight(0,CTRL_LSTICK) | CellInput->WasAnalogPressedLeft(0,CTRL_LSTICK) | CellInput->WasButtonPressed(0, CTRL_LEFT) | CellInput->WasButtonPressed(0, CTRL_CROSS))
						   {
							   Settings.DisableGraphicWindows = !Settings.DisableGraphicWindows;
						   }
						   if(CellInput->IsButtonPressed(0, CTRL_START))
						   {
							   Settings.DisableGraphicWindows = FALSE;
						   }
						   break;
					case SETTING_DISPLAY_PRESSED_KEYS:
						   if(CellInput->WasButtonPressed(0, CTRL_RIGHT) | CellInput->WasAnalogPressedRight(0, CTRL_LSTICK) | CellInput->WasAnalogPressedLeft(0, CTRL_LSTICK) | CellInput->WasButtonPressed(0, CTRL_LEFT) | CellInput->WasButtonPressed(0, CTRL_CROSS))
						   {
							   Settings.DisplayPressedKeys = !Settings.DisplayPressedKeys;
						   }
						   if(CellInput->IsButtonPressed(0, CTRL_START))
						   {
							   Settings.DisplayPressedKeys = FALSE;
						   }
						   break;
					case SETTING_FORCE_NTSC:
						   if(CellInput->WasButtonPressed(0, CTRL_RIGHT) | CellInput->WasAnalogPressedRight(0, CTRL_LSTICK) | CellInput->WasAnalogPressedLeft(0, CTRL_LSTICK) | CellInput->WasButtonPressed(0, CTRL_LEFT) | CellInput->WasButtonPressed(0, CTRL_CROSS))
						   {
							   Settings.ForceNTSC = !Settings.ForceNTSC;
						   }
						   if(CellInput->IsButtonPressed(0, CTRL_START))
						   {
							   Settings.ForceNTSC = FALSE;
						   }
						   break;
					case SETTING_PAL_TIMING:
						   if(CellInput->WasButtonPressed(0, CTRL_LEFT) | CellInput->WasAnalogPressedLeft(0,CTRL_LSTICK))
						   {
							   Settings.FrameTimePAL = 16667;
						   }
						   if(CellInput->WasButtonPressed(0, CTRL_RIGHT) | CellInput->WasAnalogPressedRight(0,CTRL_LSTICK))
						   {
							   Settings.FrameTimePAL = 20000;
						   }
						   if(CellInput->IsButtonPressed(0, CTRL_START))
						   {
							   Settings.FrameTimePAL = 20000;
						   }
						   break;
					case SETTING_DEFAULT_SNES9X_ALL:
						if(CellInput->WasButtonPressed(0, CTRL_LEFT) | CellInput->WasAnalogPressedLeft(0,CTRL_LSTICK) | CellInput->WasButtonPressed(0, CTRL_RIGHT) | CellInput->WasAnalogPressedRight(0,CTRL_LSTICK) | CellInput->IsButtonPressed(0, CTRL_START) | CellInput->WasButtonPressed(0, CTRL_CROSS))
					{
						Settings.DisplayFrameRate = FALSE;
						Settings.SoundInputRate = 31942;
						Settings.Transparency = TRUE;
						Settings.SkipFrames = AUTO_FRAMERATE;
						Settings.DisableGraphicWindows = FALSE;
						Settings.DisplayPressedKeys = FALSE;
						Settings.ForceNTSC = FALSE;
						Settings.FrameTimePAL = 20000;
						Settings.ShutdownMaster = FALSE;
					}
					break;
				default:
					break;
			} // end of switch

	//zero the screen buffer 
	Graphics_ClearSurface();

	cellDbgFontPuts		(0.05f,	0.05f,	FONT_SIZE,	GREEN,	"GENERAL");
	Graphics_RenderDbgFont();
	cellDbgFontPuts		(0.25f,	0.05f,	FONT_SIZE,	RED,	"SNES9x");
	Graphics_RenderDbgFont();
	cellDbgFontPuts		(0.45f,	0.05f,	FONT_SIZE,	GREEN,	"PATHS");
	Graphics_RenderDbgFont();

	cellDbgFontPuts		(0.05f,	0.09f,	FONT_SIZE,	currently_selected_snes9x_setting == SETTING_DISPLAY_FRAMERATE ? YELLOW : WHITE,	"Display framerate");
	Graphics_RenderDbgFont();
	cellDbgFontPuts		(0.5f,	0.09f,	FONT_SIZE,	Settings.DisplayFrameRate == FALSE ? GREEN : RED, Settings.DisplayFrameRate == TRUE ? "ON" : "OFF");
	Graphics_RenderDbgFont();

	cellDbgFontPuts		(0.05f,	0.13f,	FONT_SIZE,	currently_selected_snes9x_setting == SETTING_SOUND_INPUT_RATE ? YELLOW : WHITE,	"Sound inputrate");
	Graphics_RenderDbgFont();
	cellDbgFontPrintf	(0.5f,	0.13f,	FONT_SIZE,	Settings.SoundInputRate == 31942 ? GREEN : RED, "%d", Settings.SoundInputRate);

	Graphics_RenderDbgFont();

	cellDbgFontPuts		(0.05f,	0.17f,	FONT_SIZE,	currently_selected_snes9x_setting == SETTING_TRANSPARENCY ? YELLOW : WHITE, "Transparency effects");
	Graphics_RenderDbgFont();
	cellDbgFontPuts		(0.5f,	0.17f,	FONT_SIZE,	Settings.Transparency == TRUE ? GREEN : RED, Settings.Transparency == TRUE ? "ON" : "OFF");
	Graphics_RenderDbgFont();

	cellDbgFontPuts		(0.05f,	0.21f,	FONT_SIZE,	currently_selected_snes9x_setting == SETTING_SKIP_FRAMES ? YELLOW : WHITE,	"Frame skip");
	Graphics_RenderDbgFont();
	if(Settings.SkipFrames == AUTO_FRAMERATE)
	{
		cellDbgFontPuts(0.5f, 0.21f, FONT_SIZE, GREEN, "AUTO");
		Graphics_RenderDbgFont();
	}
	else {
		cellDbgFontPrintf(0.5f, 0.21f, FONT_SIZE, RED, "%d", Settings.SkipFrames);
		Graphics_RenderDbgFont();
	}

	cellDbgFontPuts		(0.05f,	0.25f,	FONT_SIZE,	currently_selected_snes9x_setting == SETTING_DISABLE_GRAPHIC_WINDOWS ? YELLOW : WHITE,	"Disable graphic windows");
	Graphics_RenderDbgFont();
	cellDbgFontPuts(0.5f,  0.25f, FONT_SIZE, Settings.DisableGraphicWindows == FALSE ? GREEN : RED, Settings.DisableGraphicWindows == TRUE ? "ON" : "OFF");
	Graphics_RenderDbgFont();

	cellDbgFontPuts(0.05f, 0.29f, FONT_SIZE, currently_selected_snes9x_setting == SETTING_DISPLAY_PRESSED_KEYS ? YELLOW : WHITE, "Display button input");
	Graphics_RenderDbgFont();
	cellDbgFontPuts(0.5f,  0.29f, FONT_SIZE, Settings.DisplayPressedKeys == FALSE ? GREEN : RED, Settings.DisplayPressedKeys == TRUE ? "ON" : "OFF");
	Graphics_RenderDbgFont();

	cellDbgFontPuts(0.05f, 0.33f, FONT_SIZE, currently_selected_snes9x_setting == SETTING_FORCE_NTSC ? YELLOW : WHITE, "Force PAL ROM to NTSC");
	Graphics_RenderDbgFont();
	cellDbgFontPuts(0.5f,  0.33f, FONT_SIZE, Settings.ForceNTSC == FALSE ? GREEN : RED, Settings.ForceNTSC == TRUE ? "ON" : "OFF");
	Graphics_RenderDbgFont();

	cellDbgFontPuts(0.05f, 0.37f, FONT_SIZE, currently_selected_snes9x_setting == SETTING_PAL_TIMING ? YELLOW : WHITE, "PAL Timing");
	Graphics_RenderDbgFont();
	cellDbgFontPrintf(0.5f, 0.37f, FONT_SIZE, Settings.FrameTimePAL == 20000 ? GREEN : RED, "%d", Settings.FrameTimePAL);
	Graphics_RenderDbgFont();

	cellDbgFontPrintf(0.05f, 0.41f, FONT_SIZE, currently_selected_snes9x_setting == SETTING_DEFAULT_SNES9X_ALL ? YELLOW : GREEN, "DEFAULT");
	Graphics_RenderDbgFont();


	cellDbgFontPuts(0.01f, 0.92f, FONT_SIZE, YELLOW, "UP/DOWN - select  LEFT/RIGHT - change  START - default  L1/CIRCLE - go back");
	Graphics_RenderDbgFont();
	cellDbgFontPuts(0.01f, 0.96f, FONT_SIZE, YELLOW, "R1 - go forward");
	Graphics_RenderDbgFont();


	cellSysutilCheckCallback();
	Graphics_Render(true);
}
// void do_general_settings()
// // called from ROM menu by pressing the SELECT button
// // return to ROM menu by pressing the CIRCLE button
int currently_selected_setting = 0;
void do_general_settings()
{
	CellVideoOutState video_state;
	int current_resolution;
	if(initial_resolution == 0)
	{
		video_state = Graphics_GetVideoState();
		initial_resolution = video_state.displayMode.resolutionId;
	}


	CellInput->UpdateDevice(0);
			// back to ROM menu if CIRCLE is pressed
			if (CellInput->WasButtonPressed(0, CTRL_CIRCLE))
			{
				menuStack.pop();
				return;
			}

			if (CellInput->WasButtonPressed(0, CTRL_R1))
			{
				menuStack.push(do_snes9x_settings);
				return;
			}

			if (CellInput->WasButtonPressed(0, CTRL_DOWN) | CellInput->WasAnalogPressedDown(0, CTRL_LSTICK))	// down to next setting
			{
				currently_selected_setting++;
				if (currently_selected_setting >= MAX_NO_OF_SETTINGS)
				{
					currently_selected_setting = 0;
				}
			}

			if (CellInput->WasButtonPressed(0, CTRL_UP) | CellInput->WasAnalogPressedUp(0, CTRL_LSTICK))	// up to previous setting
			{
					currently_selected_setting--;
					if (currently_selected_setting < 0)
					{
						currently_selected_setting = MAX_NO_OF_SETTINGS-1;
					}
			}
					switch(currently_selected_setting)
					{
					case SETTING_CURRENT_SAVE_STATE_SLOT:
						   if(CellInput->WasButtonPressed(0, CTRL_LEFT) | CellInput->WasAnalogPressedLeft(0,CTRL_LSTICK) | CellInput->WasButtonPressed(0, CTRL_CROSS))
						   {
							   Emulator_DecrementCurrentSaveStateSlot();
						   }
						   if(CellInput->WasButtonPressed(0, CTRL_RIGHT) | CellInput->WasAnalogPressedRight(0,CTRL_LSTICK) | CellInput->WasButtonPressed(0, CTRL_CROSS))
						   {
							   Emulator_IncrementCurrentSaveStateSlot();
						   }
						   if(CellInput->IsButtonPressed(0, CTRL_START))
						   {
							   Emulator_SetCurrentSaveStateSlot(0);
						   }
						   break;
					case SETTING_CHANGE_RESOLUTION:
#ifdef PS3_SDK_3_41
						   cellVideoOutGetState(CELL_VIDEO_OUT_PRIMARY, 0, &video_state);
						   current_resolution = video_state.displayMode.resolutionId;
						   if(CellInput->WasButtonPressed(0, CTRL_RIGHT) | CellInput->WasAnalogPressedLeft(0,CTRL_LSTICK) | CellInput->WasButtonPressed(0,CTRL_CROSS))
						   {
							   switch(initial_resolution)
							   {
								   case CELL_VIDEO_OUT_RESOLUTION_480:
									   break;
								  case CELL_VIDEO_OUT_RESOLUTION_720:
									  break;
								  case CELL_VIDEO_OUT_RESOLUTION_1080:
									  if (current_resolution == CELL_VIDEO_OUT_RESOLUTION_720)
									  {
										  if (Graphics_ChangeResolution(CELL_VIDEO_OUT_RESOLUTION_1080, CELL_VIDEO_OUT_REFRESH_RATE_59_94HZ) != CELL_OK)
										  {
											  Emulator_Shutdown();
										  }
									  }
									  if (current_resolution == CELL_VIDEO_OUT_RESOLUTION_480)
									  {
										  if (Graphics_ChangeResolution(CELL_VIDEO_OUT_RESOLUTION_720, CELL_VIDEO_OUT_REFRESH_RATE_59_94HZ) != CELL_OK)
										  {
											  Emulator_Shutdown();
										  }
									  }
									  break;
								  case CELL_VIDEO_OUT_RESOLUTION_576:
									  break;
								  case CELL_VIDEO_OUT_RESOLUTION_1600x1080:
									  break;
								  case CELL_VIDEO_OUT_RESOLUTION_1440x1080:
									  break;
								  case CELL_VIDEO_OUT_RESOLUTION_1280x1080:
									  break;
								  case CELL_VIDEO_OUT_RESOLUTION_960x1080:
									  break;
							  }
						   }
						   if(CellInput->WasButtonPressed(0, CTRL_LEFT) | CellInput->WasAnalogPressedLeft(0,CTRL_LSTICK) | CellInput->WasButtonPressed(0,CTRL_CROSS))
						   {
							   switch(initial_resolution)
							   {
								   case CELL_VIDEO_OUT_RESOLUTION_480:
									   break;
								   case CELL_VIDEO_OUT_RESOLUTION_720:
									   if (current_resolution == CELL_VIDEO_OUT_RESOLUTION_720)
									   {
										   if (Graphics_ChangeResolution(CELL_VIDEO_OUT_RESOLUTION_480, CELL_VIDEO_OUT_REFRESH_RATE_59_94HZ) != CELL_OK)
										   {
											   Emulator_Shutdown();
										   }
									   }
									   break;
								   case CELL_VIDEO_OUT_RESOLUTION_1080:
									   if (current_resolution == CELL_VIDEO_OUT_RESOLUTION_720)
									   {
										   if (Graphics_ChangeResolution(CELL_VIDEO_OUT_RESOLUTION_480, CELL_VIDEO_OUT_REFRESH_RATE_59_94HZ) != CELL_OK)
										   {
											   Emulator_Shutdown();
										   }
									   }
									   if (current_resolution == CELL_VIDEO_OUT_RESOLUTION_1080)
									   {
										   if (Graphics_ChangeResolution(CELL_VIDEO_OUT_RESOLUTION_720, CELL_VIDEO_OUT_REFRESH_RATE_59_94HZ) != CELL_OK)
										   {
											   Emulator_Shutdown();
										   }
									   }
									   break;
								   case CELL_VIDEO_OUT_RESOLUTION_576:
									   break;
								   case CELL_VIDEO_OUT_RESOLUTION_1600x1080:
									   break;
								   case CELL_VIDEO_OUT_RESOLUTION_1440x1080:
									   break;
								   case CELL_VIDEO_OUT_RESOLUTION_1280x1080:
									   break;
								   case CELL_VIDEO_OUT_RESOLUTION_960x1080:
									   break;
							   }
						   }
						   if(CellInput->IsButtonPressed(0, CTRL_START))
						   {
							   if(Graphics_ChangeResolution(initial_resolution, CELL_VIDEO_OUT_REFRESH_RATE_59_94HZ) != CELL_OK)
							   {
								   Emulator_Shutdown();
							   }

						   }
#endif
						   break;
				case SETTING_KEEP_ASPECT_RATIO:
					cellVideoOutGetState(CELL_VIDEO_OUT_PRIMARY, 0, &video_state);
					if(CellInput->WasButtonPressed(0, CTRL_LEFT) | CellInput->WasAnalogPressedLeft(0,CTRL_LSTICK) | CellInput->WasButtonPressed(0, CTRL_RIGHT) | CellInput->WasAnalogPressedRight(0,CTRL_LSTICK) | CellInput->WasButtonPressed(0,CTRL_CROSS))
					{
						Graphics_SetAspectRatio(!Settings.PS3KeepAspect);
					}
					if(CellInput->IsButtonPressed(0, CTRL_START))
					{
						Settings.PS3KeepAspect = TRUE;
						Graphics_SetAspectRatio(Settings.PS3KeepAspect);
					}
					break;
				case SETTING_HW_TEXTURE_FILTER:
					if(CellInput->WasButtonPressed(0, CTRL_LEFT) | CellInput->WasAnalogPressedLeft(0,CTRL_LSTICK) | CellInput->WasButtonPressed(0, CTRL_RIGHT) | CellInput->WasAnalogPressedRight(0,CTRL_LSTICK) | CellInput->WasButtonPressed(0,CTRL_CROSS))
					{
						Graphics_SetTextureFilter(!Settings.PS3Smooth);
					}
					if(CellInput->IsButtonPressed(0, CTRL_START))
					{
						Settings.PS3Smooth = TRUE;
						Graphics_SetTextureFilter(Settings.PS3Smooth);
					}
					break;
				case SETTING_HW_OVERSCAN_AMOUNT:
					if(CellInput->WasButtonPressed(0, CTRL_LEFT) | CellInput->WasAnalogPressedLeft(0,CTRL_LSTICK) | CellInput->WasButtonPressed(0,CTRL_CROSS))
					{
						if(Settings.PS3OverscanAmount > -10)
						{
							Settings.PS3OverscanAmount--;
							Settings.PS3OverscanEnabled = TRUE;
							Graphics_SetOverscan(Settings.PS3OverscanEnabled, (float)Settings.PS3OverscanAmount/100);
						}
						if(Settings.PS3OverscanAmount == 0)
						{
							Settings.PS3OverscanEnabled = FALSE;
							Graphics_SetOverscan(Settings.PS3OverscanEnabled, (float)Settings.PS3OverscanAmount/100);
						}
					}
					if(CellInput->WasButtonPressed(0, CTRL_RIGHT) | CellInput->WasAnalogPressedRight(0,CTRL_LSTICK) | CellInput->WasButtonPressed(0,CTRL_CROSS))
					{
						if((Settings.PS3OverscanAmount < 10))
						{
							Settings.PS3OverscanAmount++;
							Settings.PS3OverscanEnabled = TRUE;
							Graphics_SetOverscan(Settings.PS3OverscanEnabled, (float)Settings.PS3OverscanAmount/100);
						}
						if(Settings.PS3OverscanAmount == 0)
						{
							Settings.PS3OverscanEnabled = FALSE;
							Graphics_SetOverscan(Settings.PS3OverscanEnabled, (float)Settings.PS3OverscanAmount/100);
						}
					}
					if(CellInput->IsButtonPressed(0, CTRL_START))
					{
						Settings.PS3OverscanAmount = 0;
						Settings.PS3OverscanEnabled = FALSE;
						Graphics_SetOverscan(Settings.PS3OverscanEnabled, (float)Settings.PS3OverscanAmount/100);
					}
					break;
				case SETTING_RSOUND_ENABLED:
					if(CellInput->WasButtonPressed(0, CTRL_LEFT) | CellInput->WasAnalogPressedLeft(0,CTRL_LSTICK) | CellInput->WasButtonPressed(0, CTRL_RIGHT) | CellInput->WasAnalogPressedRight(0,CTRL_LSTICK) | CellInput->WasButtonPressed(0,CTRL_CROSS))
					{
						Settings.RSoundEnabled = !Settings.RSoundEnabled;
						Emulator_ToggleSound();
					}
					if(CellInput->IsButtonPressed(0, CTRL_START))
					{
						Settings.RSoundEnabled = FALSE;
						Emulator_ToggleSound();
					}
					break;
				case SETTING_RSOUND_SERVER_IP_ADDRESS:
					if(CellInput->WasButtonPressed(0, CTRL_LEFT) | CellInput->WasAnalogPressedLeft(0,CTRL_LSTICK) | CellInput->WasButtonPressed(0, CTRL_RIGHT) | CellInput->WasButtonPressed(0, CTRL_CROSS) | CellInput->WasAnalogPressedRight(0,CTRL_LSTICK))
					{
						CellInput->oskutil.Start(L"Enter the IP address for the RSound Server. Example (below):", L"192.168.1.1");
						Settings.RSoundServerIPAddress = CellInput->oskutil.OutputString();
					}
					if(CellInput->IsButtonPressed(0, CTRL_START))
					{
						Settings.RSoundServerIPAddress = "0.0.0.0";
					}
					break;
				case SETTING_DEFAULT_ALL:
					if(CellInput->WasButtonPressed(0, CTRL_LEFT) | CellInput->WasAnalogPressedLeft(0,CTRL_LSTICK) | CellInput->WasButtonPressed(0, CTRL_RIGHT) | CellInput->WasAnalogPressedRight(0,CTRL_LSTICK) | CellInput->IsButtonPressed(0, CTRL_START) | CellInput->WasButtonPressed(0, CTRL_CROSS))
					{
						Settings.PS3KeepAspect = TRUE;
						Settings.PS3Smooth = TRUE;
						Settings.RSoundEnabled = FALSE;
						Settings.PS3OverscanEnabled = FALSE;
						Settings.PS3OverscanAmount = 0;
						Settings.RSoundServerIPAddress = "0.0.0.0";
						Graphics_SetAspectRatio(Settings.PS3KeepAspect);
						Graphics_SetTextureFilter(Settings.PS3Smooth);
						Graphics_SetOverscan(Settings.PS3OverscanEnabled, (float)Settings.PS3OverscanAmount/100);
						Emulator_SetCurrentSaveStateSlot(0);
						Emulator_ToggleSound();
					}
					break;
				default:
					break;
			} // end of switch

	//zero the screen buffer 
	Graphics_ClearSurface();

	cellDbgFontPuts		(0.05f,	0.05f,	FONT_SIZE,	RED,	"GENERAL");
	Graphics_RenderDbgFont();
	cellDbgFontPuts		(0.25f,	0.05f,	FONT_SIZE,	GREEN,	"SNES9x");
	Graphics_RenderDbgFont();
	cellDbgFontPuts		(0.45f,	0.05f,	FONT_SIZE,	GREEN,	"PATHS");
	Graphics_RenderDbgFont();

	cellDbgFontPuts(0.05f, 0.09f, FONT_SIZE, currently_selected_setting == SETTING_CURRENT_SAVE_STATE_SLOT ? YELLOW : WHITE, "Current save state slot");
	Graphics_RenderDbgFont();
	cellDbgFontPrintf(0.5f, 0.09f, FONT_SIZE, Emulator_CurrentSaveStateSlot() == 0 ? GREEN : RED, "%d", Emulator_CurrentSaveStateSlot());
	Graphics_RenderDbgFont();

	cellDbgFontPuts(0.05f, 0.13f, FONT_SIZE, currently_selected_setting == SETTING_CHANGE_RESOLUTION ? YELLOW : WHITE, "Resolution");
	Graphics_RenderDbgFont();

	// get the video state again with new resolution
	cellVideoOutGetState(CELL_VIDEO_OUT_PRIMARY, 0, &video_state);
	switch(video_state.displayMode.resolutionId)
	{
		case CELL_VIDEO_OUT_RESOLUTION_480:
			cellDbgFontPrintf(0.5f, 0.13f, FONT_SIZE, GREEN, "480");
			Graphics_RenderDbgFont();
			break;
		case CELL_VIDEO_OUT_RESOLUTION_720:
			cellDbgFontPrintf(0.5f, 0.13f, FONT_SIZE, GREEN, "720");
			Graphics_RenderDbgFont();
			break;
		case CELL_VIDEO_OUT_RESOLUTION_1080:
			cellDbgFontPrintf(0.5f, 0.13f, FONT_SIZE, GREEN, "1080");
			Graphics_RenderDbgFont();
			break;
		case CELL_VIDEO_OUT_RESOLUTION_576:
			cellDbgFontPrintf(0.5f, 0.13f, FONT_SIZE, GREEN, "576");
			Graphics_RenderDbgFont();
			break;
		case CELL_VIDEO_OUT_RESOLUTION_1600x1080:
			cellDbgFontPrintf(0.5f, 0.13f, FONT_SIZE, GREEN, "1600x1080");
			Graphics_RenderDbgFont();
			break;
		case CELL_VIDEO_OUT_RESOLUTION_1440x1080:
			cellDbgFontPrintf(0.5f, 0.13f, FONT_SIZE, GREEN, "1440x1080");
			Graphics_RenderDbgFont();
			break;
		case CELL_VIDEO_OUT_RESOLUTION_1280x1080:
			cellDbgFontPrintf(0.5f, 0.13f, FONT_SIZE, GREEN, "1280x1080");
			Graphics_RenderDbgFont();
			break;
		case CELL_VIDEO_OUT_RESOLUTION_960x1080:
			cellDbgFontPrintf(0.5f, 0.13f, FONT_SIZE, GREEN, "960x1080");
			Graphics_RenderDbgFont();
			break;
	}

	cellDbgFontPuts(0.05f, 0.17f, FONT_SIZE, currently_selected_setting == SETTING_KEEP_ASPECT_RATIO ? YELLOW : WHITE, "Aspect Ratio");
	Graphics_RenderDbgFont();
#ifdef PSGL
	cellDbgFontPrintf(0.5f, 0.17f, FONT_SIZE, Settings.PS3KeepAspect == true ? GREEN : RED, "%s", Settings.PS3KeepAspect == true ? "4:3" : "16:9");
	Graphics_RenderDbgFont();
#endif

	cellDbgFontPuts(0.05f, 0.21f, FONT_SIZE, currently_selected_setting == SETTING_HW_TEXTURE_FILTER ? YELLOW : WHITE, "Hardware Filtering");
	Graphics_RenderDbgFont();
#ifdef PSGL
	cellDbgFontPrintf(0.5f, 0.21f, FONT_SIZE, Settings.PS3Smooth == true ? GREEN : RED,
			"%s", Settings.PS3Smooth == true ? "Linear interpolation" : "Point filtering");
	Graphics_RenderDbgFont();
#endif

	cellDbgFontPuts		(0.05f,	0.25f,	FONT_SIZE,	currently_selected_setting == SETTING_HW_OVERSCAN_AMOUNT ? YELLOW : WHITE,	"Overscan");
	Graphics_RenderDbgFont();
	cellDbgFontPrintf	(0.5f,	0.25f,	FONT_SIZE,	Settings.PS3OverscanAmount == 0 ? GREEN : RED, "%f", (float)Settings.PS3OverscanAmount/100);

	Graphics_RenderDbgFont();

	cellDbgFontPuts(0.05f, 0.29f, FONT_SIZE, currently_selected_setting == SETTING_RSOUND_ENABLED ? YELLOW : WHITE, "Sound");
	Graphics_RenderDbgFont();
	cellDbgFontPuts(0.5f, 0.29f, FONT_SIZE, Settings.RSoundEnabled == FALSE ? GREEN : RED, Settings.RSoundEnabled == TRUE ? "RSound" : "Normal");
	Graphics_RenderDbgFont();

	cellDbgFontPuts(0.05f, 0.33f, FONT_SIZE, currently_selected_setting == SETTING_RSOUND_SERVER_IP_ADDRESS ? YELLOW : WHITE, "RSound Server IP Address");
	Graphics_RenderDbgFont();
	cellDbgFontPuts(0.5f, 0.33f, FONT_SIZE, strcmp(Settings.RSoundServerIPAddress,"0.0.0.0") ? RED : GREEN, Settings.RSoundServerIPAddress);
	Graphics_RenderDbgFont();

	cellDbgFontPrintf(0.05f, 0.37f, FONT_SIZE, currently_selected_setting == SETTING_DEFAULT_ALL ? YELLOW : GREEN, "DEFAULT");
	Graphics_RenderDbgFont();


	cellDbgFontPuts(0.01f, 0.92f, FONT_SIZE, YELLOW, "UP/DOWN - select  LEFT/RIGHT - change  START - default  CIRCLE - return to menu");
	Graphics_RenderDbgFont();
	cellDbgFontPuts(0.01f, 0.96f, FONT_SIZE, YELLOW, "R1 - go forward");
	Graphics_RenderDbgFont();


	cellSysutilCheckCallback();
	Graphics_Render(true);
}


// FIXME: I hate seeing variables declared like this.  This probably means it should be a struct, class or in its own file
bool needs_update = true;
std::vector<CellFsDirent> dirent(8192);
std::stack<char*> directoryStack;
const char *errormsg = NULL;
int fd;
int num_per_page = 24;
int current_index = 0;
int file_count = 0;

const char * getFileExtension(const char * filename)
{
	const char * ptr = strrchr(filename, '.');
	if (ptr)
	{
		ptr++;
	}
	else
	{
		ptr = "";
	}
	return ptr;
}


char* do_pathmenu()
{
	char pathselection[1024];
	strcpy(pathselection, "/");
	int file_count_pathmenu = 0;
	int current_index_pathmenu = 0;
	int fd_pathmenu;
	std::vector<CellFsDirent> dirent_pathmenu(8192);
	bool needs_pathmenu_update = true;

	// start the menu
	while(Emulator_ROMRunning())
	{
		if(needs_pathmenu_update)
		{
			needs_pathmenu_update = false;
			current_index_pathmenu = 0;

			int i = 0;
			if (cellFsOpendir(pathselection, &fd_pathmenu) == CELL_FS_SUCCEEDED)
			{
				uint64_t nread;
				
				while (cellFsReaddir(fd_pathmenu, &dirent_pathmenu[i], &nread) == CELL_FS_SUCCEEDED)
				{
					if (nread == 0)
					{
						break;
					}
					
					// if this entry is current directory (i.e, '.')
					// // then skip as there's no need to display it
					if (dirent_pathmenu[i].d_type == CELL_FS_TYPE_DIRECTORY && !(strcmp(dirent_pathmenu[i].d_name, ".")))
					{
						continue;
					}
					if (dirent_pathmenu[i].d_type != CELL_FS_TYPE_DIRECTORY && dirent_pathmenu[i].d_type != CELL_FS_TYPE_REGULAR)
					{
						continue; // so lazy.
					}
					
					++i;
					
					if (i == 8192)
					{
						break;
					}
				}
				cellFsClosedir(fd_pathmenu);
			}
			else
			{
				errormsg = "Directory not found. Try again!";
			}
			
			file_count_pathmenu = i;
			
			needs_pathmenu_update = false;
		}
		
		// sort directory file entries - courtesy of cmonkey69
		// start sorting from first entry after '..' and sort file_count - 2 items (indexing starts at zero, don't sort '.' and '..')
		qsort_structs(&dirent_pathmenu[1], 0, file_count_pathmenu - 2);
		
		if(CellInput->UpdateDevice(0) == CELL_PAD_OK)
		{
			if (CellInput->WasButtonPressed(0,CTRL_DOWN) | CellInput->IsAnalogPressedDown(0,CTRL_LSTICK))
			{
				current_index_pathmenu = MIN(current_index_pathmenu + 1, file_count_pathmenu - 1);
			}
			if (CellInput->WasButtonPressed(0,CTRL_UP) | CellInput->IsAnalogPressedUp(0,CTRL_LSTICK))
			{
				current_index_pathmenu = MAX(current_index_pathmenu - 1, 0);
			}
			if (CellInput->WasButtonPressed(0,CTRL_RIGHT) | CellInput->IsAnalogPressedRight(0,CTRL_LSTICK))
			{
				current_index_pathmenu = MIN(current_index_pathmenu + 5, file_count_pathmenu - 1);
			}
			if (CellInput->WasButtonPressed(0,CTRL_LEFT) | CellInput->IsAnalogPressedLeft(0,CTRL_LSTICK))
			{
				current_index_pathmenu = MAX(current_index_pathmenu - 5, 0);
			}
			if (CellInput->WasButtonPressed(0,CTRL_R1))
			{
				current_index_pathmenu = MIN(current_index_pathmenu + num_per_page, file_count_pathmenu - 1);
			}
			if (CellInput->WasButtonPressed(0,CTRL_L1))
			{
				current_index_pathmenu = MAX(current_index_pathmenu - num_per_page, 0);
			}
			if (CellInput->WasButtonPressed(0,CTRL_CROSS))
			{
				if(dirent_pathmenu[current_index_pathmenu].d_type == CELL_FS_TYPE_DIRECTORY)
				{
					sprintf(pathselection, "%s/%s", pathselection, dirent_pathmenu[current_index_pathmenu].d_name);
					needs_pathmenu_update = true;
					continue;
				}
			}
			if (CellInput->WasButtonPressed(0,CTRL_TRIANGLE))
			{
				if (dirent_pathmenu[current_index_pathmenu].d_type == CELL_FS_TYPE_DIRECTORY)
				{
					//set path selection that we selected with the triangle button
					char* newPath = (char*)malloc(sizeof(char) * MAX_PATH);
					newPath[0] = '\0';
					sprintf(newPath, "%s/%s", pathselection, dirent_pathmenu[current_index_pathmenu].d_name);
					return newPath;
				}
			}
		}
		
		Graphics_ClearSurface();
		
		// if you don't want this crap at the bottom of the screen then comment it out
		// and increase num_per_page to 26 to shows more roms per screen
		cellDbgFontPuts(0.05f, 0.88f, FONT_SIZE, YELLOW, "X - Enter directory");
		Graphics_RenderDbgFont();
		cellDbgFontPuts(0.5f, 0.88f, FONT_SIZE, CYAN ,"/\\ - Select directory as path");
		Graphics_RenderDbgFont();
		
		int page_number = current_index_pathmenu / num_per_page;
		int page_base = page_number * num_per_page;
		
		for (int i = page_base; i < file_count_pathmenu && i < page_base + num_per_page; ++i)
		{
			// libdbgfont colour is ABGR format
			// left = 0.05f, top = 0.04f, bottom = 0.925f, spacing = 0.035f, font size = FONT_SIZE
			// the above settings look good on my TV at 480p but yours may differ depending on the overscan amount of your TV
			cellDbgFontPuts(0.05f, 0.04f + 0.035f * ((float)i - (float)page_base), FONT_SIZE, i == current_index_pathmenu ? RED : dirent_pathmenu[i].d_type == CELL_FS_TYPE_DIRECTORY ? GREEN : WHITE, dirent_pathmenu[i].d_name);		
			Graphics_RenderDbgFont();
		}

		//added line to it that if you get an error message, it should set the path back to root
		//so we can at least return to the root of the filesystem rather than just show an 
		//errormessage for infinity
		if (errormsg)
		{
			cellDbgFontPuts(0.3f, 0.5f, FONT_SIZE, WHITE, errormsg);
			Graphics_RenderDbgFont();
			strcpy(pathselection, "/");
		}
		
		cellSysutilCheckCallback();
		Graphics_Render(true);

		//FIXME: ugly hack again
		//if errormsg is something other than NULL, we want it to trigger a needs_pathmenu_update again - we already set the path to the 
		//root in the previous step (see above)
		if (errormsg)
		{
			errormsg = NULL;
			needs_pathmenu_update = true;
		}
	}
}

void do_ROMMenu()
{
	if (!directoryStack.empty())
	{
		path = directoryStack.top();
	}

	// don't need this anymore
	//don't go back all the way to the root after quitting a ROM
	/*if(path[0] == '\0')
	{
		strcpy(path, "/");
	}*/

	// start the menu
	if (needs_update) {
		needs_update = false;
		current_index = 0;

		int i = 0;
		if (cellFsOpendir(path, &fd) == CELL_FS_SUCCEEDED)
		{
			uint64_t nread;

			while (cellFsReaddir(fd, &dirent[i], &nread) == CELL_FS_SUCCEEDED)
			{
				if (nread == 0)
					break;

				// if this entry is current directory (i.e, '.')
				// then skip as there's no need to display it
				if (dirent[i].d_type == CELL_FS_TYPE_DIRECTORY &&
					!(strcmp(dirent[i].d_name, ".")))
				{
					continue;
				}
				if (dirent[i].d_type != CELL_FS_TYPE_DIRECTORY &&
					dirent[i].d_type != CELL_FS_TYPE_REGULAR)
				{
					continue; // so lazy.
				}

				if (dirent[i].d_type == CELL_FS_TYPE_REGULAR)
				{
					const char * ext = getFileExtension(dirent[i].d_name);

					if (strcasecmp(ext, "smc") && strcasecmp(ext, "fig") &&
						strcasecmp(ext, "sfc") && strcasecmp(ext, "swc") &&
						strcasecmp(ext, "zip") && strcasecmp(ext, "jma"))
					{
						continue;
					}
				}

				++i;

				if (i == 8192)
				{
					break;
				}
			}

			cellFsClosedir(fd);
		} else {
			errormsg = "Directory not found. Try again!";
		}

		file_count = i;

		needs_update = false;
	}

	// sort directory file entries - courtesy of cmonkey69
	// start sorting from first entry after '..' and sort file_count - 2 items (indexing starts at zero, don't sort '.' and '..')
	qsort_structs(&dirent[1], 0, file_count - 2);

	if(CellInput->UpdateDevice(0) == CELL_PAD_OK)
	{
		if (CellInput->WasButtonPressed(0,CTRL_DOWN) | CellInput->IsAnalogPressedDown(0,CTRL_LSTICK))
		{
				current_index = MIN(current_index + 1, file_count - 1);
		}
		if (CellInput->WasButtonPressed(0,CTRL_UP) | CellInput->IsAnalogPressedUp(0,CTRL_LSTICK))
		{
				current_index = MAX(current_index - 1, 0);
		}
		if (CellInput->WasButtonPressed(0,CTRL_RIGHT) | CellInput->IsAnalogPressedRight(0,CTRL_LSTICK))
		{
				current_index = MIN(current_index + 5, file_count - 1);
		}
		if (CellInput->WasButtonPressed(0,CTRL_LEFT) | CellInput->IsAnalogPressedLeft(0,CTRL_LSTICK))
		{
				current_index = MAX(current_index - 5, 0);
		}
		if (CellInput->WasButtonPressed(0,CTRL_R1))
		{
				current_index = MIN(current_index + num_per_page, file_count - 1);
		}
		if (CellInput->WasButtonPressed(0,CTRL_L1))
		{
				current_index = MAX(current_index - num_per_page, 0);
		}
		if (CellInput->WasButtonPressed(0,CTRL_SELECT))
		{
				menuStack.push(do_general_settings);
		}
		if (CellInput->WasButtonPressed(0, CTRL_CIRCLE))
		{
			// don't let people back out past root
			if (directoryStack.size() > 1)
			{
				free(directoryStack.top());
				directoryStack.pop();
				needs_update = true;
			}
		}
		if (CellInput->WasButtonPressed(0,CTRL_CROSS))
		{
			if(dirent[current_index].d_type == CELL_FS_TYPE_DIRECTORY) {
				if((current_index == 0) && (directoryStack.size() > 1))
				{
					free(directoryStack.top());
					directoryStack.pop();
					needs_update = true;
				}
				else
				{
					char* newPath = (char*)malloc(sizeof(char) * MAX_PATH);
					newPath[0] = '\0';
					sprintf(newPath, "%s/%s", path, dirent[current_index].d_name);
					directoryStack.push(newPath);
					needs_update = true;
				}
			}
			else
			{
				//load game (standard controls), go back to main loop
				sprintf(rom_path, "%s/%s", path, dirent[current_index].d_name);

				MenuStop();

				// switch emulator to emulate mode
				Emulator_StartROMRunning();

				Emulator_SetControllerMode(TWO_JOYSTICKS);

				Emulator_RequestLoadROM(MenuGetSelectedROM(), true);

				return;
			}
		}
		if (CellInput->WasButtonPressed(0,CTRL_TRIANGLE))
		{
			if (dirent[current_index].d_type == CELL_FS_TYPE_REGULAR)
			{
				//load game with multitap support
				sprintf(rom_path, "%s/%s", path, dirent[current_index].d_name);

				// stop the menu and pop the stack
				MenuStop();

				// switch emulator to emulate mode
				Emulator_StartROMRunning();
				Emulator_SetControllerMode(MULTITAP);

				Emulator_RequestLoadROM(MenuGetSelectedROM(), true);

				return;
			}
		}
		if (CellInput->IsButtonPressed(0,CTRL_L2) & CellInput->IsButtonPressed(0,CTRL_R2))
		{
			// if a rom is loaded then resume it
			if (Emulator_IsROMLoaded())
			{
				MenuStop();

				Emulator_RequestLoadROM(MenuGetSelectedROM(), false);
				Emulator_StartROMRunning();
			}

			return;
		}
	}

	Graphics_ClearSurface();

	// if you don't want this crap at the bottom of the screen then comment it out 
	// and increase num_per_page to 26 to shows more roms per screen
	cellDbgFontPuts(0.05f, 0.88f, FONT_SIZE, YELLOW, "X - enter directory/load game");
	Graphics_RenderDbgFont();
	cellDbgFontPuts(0.5f, 0.88f, FONT_SIZE, CYAN ,"/\\ - load game with multitap support");
	Graphics_RenderDbgFont();
	cellDbgFontPuts(0.05f, 0.92f, FONT_SIZE, PURPLE, "L2 + R2 - return to game");
	Graphics_RenderDbgFont();
	cellDbgFontPuts(0.5f, 0.92f, FONT_SIZE, BLUE, "SELECT - settings screen");
	Graphics_RenderDbgFont();

	int page_number = current_index / num_per_page;
	int page_base = page_number * num_per_page;

	for (int i = page_base; i < file_count && i < page_base + num_per_page; ++i)
	{
	// libdbgfont colour is ABGR format
		// left = 0.05f, top = 0.04f, bottom = 0.925f, spacing = 0.035f, font size = FONT_SIZE
		// the above settings look good on my TV at 480p but yours may differ depending on the overscan amount of your TV
		cellDbgFontPuts(0.05f, 0.04f + 0.035f * ((float)i - (float)page_base), FONT_SIZE, 
			i == current_index ? RED : dirent[i].d_type == CELL_FS_TYPE_DIRECTORY ? GREEN : WHITE, dirent[i].d_name);		
		Graphics_RenderDbgFont();
	}

	//added line to it that if you get an error message, it should set the path back to root
	//so we can at least return to the root of the filesystem rather than just show an 
	//errormessage for infinity
	if (errormsg)
	{
		cellDbgFontPuts(0.3f, 0.5f, FONT_SIZE, WHITE, errormsg);
		Graphics_RenderDbgFont();
		strcpy(path, "/");
	}

	cellSysutilCheckCallback();
	Graphics_Render(true);

	//FIXME: ugly hack again
	//if errormsg is something other than NULL, we want it to trigger a needs_update again - we already set the path to the 
	//root in the previous step (see above)
	if (errormsg)
	{
		errormsg = NULL;
		needs_update = true;
	}

}

void MainMenu()
{
	if (directoryStack.empty())
	{
		directoryStack.push((char*)Settings.PS3PathROMDirectory);
		/*
		else
		{
			// allocate root path
			char* startPath = (char*)malloc(sizeof(char) * MAX_PATH);
			startPath[0] = '/';
			startPath[1] = '\0';
			directoryStack.push(startPath);
		}
		*/
	}

	menuStack.push(do_ROMMenu);

	menuRunning = true;
	while (!menuStack.empty() && menuRunning)
	{
		menuStack.top()();
	}
}
